## Description

This module uses twig to support SQL expressions for fields,
filters, sorts and arguments. Users can also enter SQL
expressions directly. Twig support allow more powerful
computation to be performed. 

## Warning, Please Read
This module provides access to SQL expressions within Views.
It should only be used by those who understand SQL. 
It does provide access to features that would not be possible 
such as aggregation of computed fields. 

With great power comes great responsibility and care. 
It is possible to mess up a view with expressions that
generate illegal syntax so backing up and making copies
of working views is a good idea. 

Unlike Views in general, this module does not try to protect
you from bad syntax, etc. Please do not post questions 
regarding SQL but do include those where SQL is not being
generated prooperly. 

## REQUIREMENTS
This module requires:
* Views - part of the core
* Views-UI - part of the core

## Permissions
There is a Twig SQL administration option that will prevent changes
to views that utilize Twig SQL components. It essentially turns all
the SQL fields to read-only mode. Users can view the settings but
cannot change them. 

## Usage
1. Install the module in the usual way.
2. For fields, filters, sort and argmunet: choose the SQL Twig entry 
   in the Global section.
3. For relationships (separate module) works differently. 
   It adds Twig SQL support to existing entity relationships. 
   There is a checkbox option to turn this on and off for any
   entity supported by the standard relationships. 
4. The settings can be found in the Twig settings detail. 
   The twig support is under the Twig definitions detail. 
   The SQL expression is the one added to the Views query. 
   
## How to use Twig SQL fields. 
The system can be used in either raw SQL mode or using twig. 
In the former case, the SQL expression field should contain 
a valid SQL expression used as the condition or value for 
the respective field, etc. The twig fields are ignored. 

The twig support allow substitution and twig evaluation 
that can utilize global values. It allows expressions like:
* {{ field1 }} = {{ field2 }}
* {{ tableA }}.{{ field1 }} + 12

There is a variables section where expressions can be
defined. This can be useful for constants, etc. 

## Tables, Variables and Substitutions
The Tables section provides a list of the tables 
currently in the view along with the SQL field names.
These can be copied and pasted into the substitution, 
twig or SQL areas. This is a typical use of the
substitution area as the 'table.field' names tend to be
long. Using replacements allows the expressions 
to be more concise. 

The substitution area has a 'search|replace' format.
Text in the twig area matching the search field will be
replaced. The search field can include any characters
except the separator so values like [var1] and
{{ var2 }} are valid.

This approach substition allows twig expressions to be
replacement values. 

## SQL Regeneration from Twig
The twig field will be evaluated to regenerate the SQL 
expression. It is possible to determine when the expression
is generated. For example, it is possible to generate
the SQL expression and then utilize it from then on. 
This is the recommended approach. 

Instances where regenation should occur at runtime 
are where the twig expression changes based on the time
it is evaluated. 

## Additional tokens
There are a few tokens not listed in the global token list. 
These include:
* [path:alias] aliased URL 
* [path:raw]   raw URL
* [path:route] Drupal path route name
* [path:host]  host name
* [path:parameter:$key] URL parameters

Note: the URL parameters do not work in the Views UI.
They will work for a Views view page or when a view
is on a page as a block, etc. 

## Debugging SQL
If you are knowledgable in SQL then you can probably
generate the SQL expressions the first time and get
the results you like but if you are like me you tend
to make mistakes. 

The easiest way to not get a lot of
error messages about malformed SQL expressions is to 
test the generation of the expression by placing the
expression into a string and examining the Views
preview SQL statement. This is handy since the
typical use of this module is to generate a value and 
a string is usually sufficient. For exampe: 

'{{ field1 }} = {{ field2 }}'

instead of

{{ field1 }} = {{ field2 }}

Just remove the quotes once the desired expression
is generated. 

If an error does occur then the SQL expression
will be replaced with this string.

  '== Twig SQL error =='

An error message will also be displayed with the 
expression that has the error but only it the user
has the ability to edit Twig SQL entities. 
Otherwise, the class of the object that caused
the error will be displayed. 


## MAINTAINERS
Current maintainers:
* Bill Wong (bwong) - https://www.drupal.org/u/bwong
